@@ -1,13 +1,12 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
-
+#
 #    Copyright (c) 2005-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
@@ -27,17 +26,20 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::AnyId ;
+BEGIN {
+  $Config::Model::AnyId::VERSION = '1.226';
+}
 use Config::Model::Exception ;
 use Scalar::Util qw(weaken) ;
 use warnings ;
 use Carp;
 use strict;
-our $VERSION="1.201";
-
-# use vars qw($VERSION) ;
+use Log::Log4perl qw(get_logger :levels);
 
 use base qw/Config::Model::WarpedThing/;
 
+my $logger = get_logger("Tree::Element::Id") ;
+
 # Some idea for improvement
 
 # suggest => 'foo' or '$bar foo'
@@ -52,7 +54,7 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 1.205
+version 1.226
 
 =head1 SYNOPSIS
 
@@ -112,35 +114,35 @@ sub new {
     # args hash is modified for arg check in derived class constructor
     my $args_ref = shift ; 
 
-    my $self= { } ;
+    my $self= { warning_hash => { } } ;
 
     bless $self,$type;
 
     foreach my $p (qw/element_name cargo instance config_model/) {
-	$self->{$p} = delete $args_ref->{$p} or
-	  croak "$type->new: Missing $p parameter for element ".
-	    $self->{element_name} || 'unknown' ;
+        $self->{$p} = delete $args_ref->{$p} or
+          croak "$type->new: Missing $p parameter for element ".
+            $self->{element_name} || 'unknown' ;
     }
 
     croak "$type->new: Missing cargo->type parameter for element ".
       $self->{element_name} || 'unknown' 
-	unless defined $self->{cargo}{type};
+        unless defined $self->{cargo}{type};
 
     $self->_set_parent(delete $args_ref->{parent}) ;
 
     if ($self->{cargo}{type} eq 'node') {
-	$self->{config_class_name} = delete $self->{cargo}{config_class_name}
-	  or croak  "$type->new: Missing cargo->config_class_name "
+        $self->{config_class_name} = delete $self->{cargo}{config_class_name}
+          or croak  "$type->new: Missing cargo->config_class_name "
                   . "parameter for element " 
                   . $self->{element_name} || 'unknown' ;
     }
     elsif ($self->{cargo}{type} eq 'hash' or $self->{cargo}{type} eq 'list') {
-	die "$type $self->{element_name}: using $self->{cargo}{type} will probably not work";
+        die "$type $self->{element_name}: using $self->{cargo}{type} will probably not work";
     }
 
     foreach my $p (qw/cargo_class/) {
-	next unless defined $args_ref->{$p} ;
-	$self->{$p} = delete $args_ref->{$p} ;
+        next unless defined $args_ref->{$p} ;
+        $self->{$p} = delete $args_ref->{$p} ;
     }
 
     return $self ;
@@ -221,7 +223,7 @@ or C<< default_keys => ['foo', 'bar'] >>.
 To perform special set-up on children nodes you can also use 
 
    default_with_init =>  { 'foo' => 'X=Av Y=Bv'  ,
-		           'bar' => 'Y=Av Z=Cv' }
+                           'bar' => 'Y=Av Z=Cv' }
 
 
 =item follow_keys_from
@@ -245,6 +247,12 @@ by C<allow_keys_from> specified the authorized keys for this hash.
 
   allow_keys_from => '- another_hash'
 
+=item allow_keys_matching
+
+Keys must match the specified regular expression. For instance:
+
+  allow_keys_matching => '^foo\d\d$'
+
 =item auto_create_keys
 
 When set, the default parameter (or set of parameters) are used as
@@ -253,6 +261,13 @@ keys hashes and created automatically. (valid only for hash elements)
 Called with C<< auto_create => 'foo' >>, or 
 C<< auto_create => ['foo', 'bar'] >>.
 
+=item warn_if_key_match
+
+Issue a warning if the key matches the specified regular expression
+
+=item warn_unless_key_match
+
+Issue a warning unless the key matches the specified regular expression
 
 =item auto_create_ids
 
@@ -266,17 +281,6 @@ See L</"Warp: dynamic value configuration"> below.
 
 =back
 
-=head1 About checking value
-
-By default, value checking is done while setting or reading a value.
-
-You can use 
-L<push_no_value_check()|Config::Model::Instance/"push_no_value_check ( [fetch] , [store], [type] )">
-or 
-L<pop_no_value_check()|Config::Model::Instance/"pop_no_value_check()">
-from L<Config::Model::Instance>
-to modify this behavior.
-
 =head1 Warp: dynamic value configuration
 
 The Warp functionality enables an L<HashId|Config::Model::HashId> or
@@ -356,7 +360,8 @@ leads to a nb of items greater than the max_nb constraint.
 
 my @common_params =  qw/min_index max_index max_nb default_with_init default_keys
                         follow_keys_from auto_create_ids auto_create_keys
-                        allow_keys allow_keys_from/ ;
+                        allow_keys allow_keys_from allow_keys_matching
+                        warn_if_key_match warn_unless_key_match/ ;
 
 my @allowed_warp_params = (@common_params,qw/experience level/) ;
 
@@ -371,65 +376,65 @@ sub set_properties {
 
     my %args = (%{$self->{backup}},@_) ;
 
-    print $self->name," set_properties called with @_\n" if $::debug;
+    $logger->debug($self->name," set_properties called with @_");
 
     map { $self->{$_} =  delete $args{$_} if defined $args{$_} }
       @common_params ;
 
     Config::Model::Exception::Model
-	->throw (
-		 object => $self,
-		 error => "Undefined index_type"
-		) unless defined $self->{index_type};
+        ->throw (
+                 object => $self,
+                 error => "Undefined index_type"
+                ) unless defined $self->{index_type};
 
     Config::Model::Exception::Model
-	->throw (
-		 object => $self,
-		 error => "Unexpected index_type $self->{index_type}"
-		) unless ($self->{index_type} eq 'integer' or 
-			  $self->{index_type} eq 'string');
+        ->throw (
+                 object => $self,
+                 error => "Unexpected index_type $self->{index_type}"
+                ) unless ($self->{index_type} eq 'integer' or 
+                          $self->{index_type} eq 'string');
 
     my @current_idx = $self->_get_all_indexes( );
     if (@current_idx) {
-	my $first_idx = shift @current_idx ;
-	my $last_idx  = pop   @current_idx ;
+        my $first_idx = shift @current_idx ;
+        my $last_idx  = pop   @current_idx ;
 
-	foreach my $idx ( ($first_idx, $last_idx)) {
-	    my $ok = $self->check($first_idx) ;
-	    next if $ok ;
+        foreach my $idx ( ($first_idx, $last_idx)) {
+            my $ok = $self->check($first_idx) ;
+            next if $ok ;
 
-	    # here a user input may trigger an exception even if fetch
-	    # or set value check is disabled. That's mostly because,
-	    # we cannot enforce more strict settings without random
-	    # deletion of data. For instance, if a hash contains 5
-	    # items and the max_nb of items is reduced to 3. Which 2
-	    # items should we remove ?
+            # here a user input may trigger an exception even if fetch
+            # or set value check is disabled. That's mostly because,
+            # we cannot enforce more strict settings without random
+            # deletion of data. For instance, if a hash contains 5
+            # items and the max_nb of items is reduced to 3. Which 2
+            # items should we remove ?
 
-	    # Since we cannot choose, we must raise an exception in
-	    # all cases.
-	    Config::Model::Exception::WrongValue 
-		-> throw (
-			  error => "Error while setting id property:".
-			  join("\n\t",@{$self->{error}}),
-			  object => $self
-			 ) ;
-	}
+            # Since we cannot choose, we must raise an exception in
+            # all cases.
+            Config::Model::Exception::WrongValue 
+                -> throw (
+                          error => "Error while setting id property:".
+                          join("\n\t",@{$self->{error}}),
+                          object => $self
+                         ) ;
+        }
     }
 
     if (defined $self->{auto_create_keys} or defined $self->{auto_create_ids}) {
-	$self->auto_create_elements ;
+        $self->auto_create_elements ;
     }
 
     $self->{current} = { level      => $args{level} ,
-			 experience => $args{experience}
-		       } ;
+                         experience => $args{experience}
+                       } ;
     $self->SUPER::set_parent_element_property(\%args) ;
 
     Config::Model::Exception::Model
-	->throw (
-		 object => $self,
-		 error => "Unexpected parameters :". join(' ', keys %args)
-		) if scalar keys %args ;
+        ->throw (
+                 object => $self,
+                 error => "Unexpected parameters: ". join(' ', keys %args)
+                ) if scalar keys %args ;
 }
 
 # this method will overide setting comings from a value (or maybe a
@@ -446,11 +451,11 @@ sub set_parent_element_property {
       if defined $cur->{experience} ;
 
     if (    defined $cur->{level} 
-	and ( not defined $arg_ref->{level} 
-	      or $arg_ref->{level} ne 'hidden'
-	    )
+        and ( not defined $arg_ref->{level} 
+              or $arg_ref->{level} ne 'hidden'
+            )
        ) {
-	$arg_ref->{level} = $cur->{level} ;
+        $arg_ref->{level} = $cur->{level} ;
     }
 
     $self->SUPER::set_parent_element_property($arg_ref) ;
@@ -493,7 +498,7 @@ object (as declared in the model unless they were warped):
 
 for my $datum (qw/min_index max_index max_nb index_type default_keys default_with_init
                   follow_keys_from auto_create_keys auto_create_ids
-                  morph ordered
+                  morph ordered warn_unless_key_match warn_if_key_match
                   config_model/) {
     no strict "refs";       # to register new methods in package
     *$datum = sub {
@@ -554,19 +559,19 @@ sub safe_typed_grab {
 
   my $res = eval {
     $self->grab(step => $self->{$param},
-		type => $self->get_type,
-	       ) ;
+                type => $self->get_type,
+               ) ;
   };
 
   if ($@) {
     my $e = $@ ;
     my $msg = $e ? $e->full_message : '' ;
     Config::Model::Exception::Model
-	-> throw (
-		  object => $self,
-		  error => "'$param' parameter: "
-		  . $msg
-		 ) ;
+        -> throw (
+                  object => $self,
+                  error => "'$param' parameter: "
+                  . $msg
+                 ) ;
   }
 
   return $res ;
@@ -584,8 +589,8 @@ sub get_default_keys {
     my $self = shift ;
 
     if ($self->{follow_keys_from}) {
-	my $followed = $self->safe_typed_grab('follow_keys_from') ;
-	return [ $followed -> get_all_indexes ];
+        my $followed = $self->safe_typed_grab('follow_keys_from') ;
+        return [ $followed -> get_all_indexes ];
     }
 
     my @res ;
@@ -641,7 +646,7 @@ sub handle_args {
     $self->set_properties(%args) if defined $self->{index_type} ;
 
     if (defined $warp_info) {
-	$self->check_warp_args( \@allowed_warp_params, $warp_info) ;
+        $self->check_warp_args( \@allowed_warp_params, $warp_info) ;
     }
 
     $self->submit_to_warp($self->{warp}) if $self->{warp} ;
@@ -649,39 +654,38 @@ sub handle_args {
     return $self ;
 }
 
+my %check_dispatch = map { ($_ => 'check_'.$_) ;}
+    qw/follow_keys_from allow_keys allow_keys_from allow_keys_matching
+        warn_if_key_match warn_unless_key_match/;
+
 # internal function to check the validity of the index
 sub check {
     my ($self,$idx) = @_ ; 
 
-    my @error  ;
+    Config::Model::Exception::Internal
+        -> throw (
+                  object => $self,
+                  error => "check method: key or index is not defined"
+                 ) unless defined $idx ;
 
-    if ($self->{follow_keys_from}) {
-	$self->check_follow_keys_from($idx) or return 0 ;
-    }
+    my @error ;
+    my @warn ;
 
-    if ($self->{allow_keys}) {
-	$self->check_allow_keys($idx) or return 0 ;
-    }
-
-    if ($self->{allow_keys_from}) {
-	$self->check_allow_keys_from($idx) or return 0 ;
+    foreach my $key_check_name (keys %check_dispatch) {
+        next unless $self->{$key_check_name} ;
+        my $method = $check_dispatch{$key_check_name} ;
+        $self->$method($idx,\@error,\@warn) ;
     }
 
     my $nb =  $self->fetch_size ;
     my $new_nb = $nb ;
     $new_nb++ unless $self->_exists($idx) ;
 
-    Config::Model::Exception::Internal
-	-> throw (
-		  object => $self,
-		  error => "check method: key or index is not defined"
-		 ) unless defined $idx ;
-
     if ($idx eq '') {
         push @error,"Index is empty";
     }
     elsif ($self->{index_type} eq 'integer' and $idx =~ /\D/) {
-	push @error,"Index is not integer ($idx)";
+        push @error,"Index is not integer ($idx)";
     }
     elsif (defined $self->{max_index} and $idx > $self->{max_index}) {
         push @error,"Index $idx > max_index limit $self->{max_index}" ;
@@ -692,98 +696,123 @@ sub check {
 
     push @error,"Too many instances ($new_nb) limit $self->{max_nb}, ".
       "rejected id '$idx'"
-	if defined $self->{max_nb} and $new_nb > $self->{max_nb};
+        if defined $self->{max_nb} and $new_nb > $self->{max_nb};
 
     if (scalar @error) {
-	my @a = $self->get_all_indexes ;
+        my @a = $self->get_all_indexes ;
         push @error, "Instance ids are '".join(',', @a)."'" ,
           $self->warp_error  ;
     }
 
     $self->{error} = \@error ;
-    return not scalar @error ;
+
+    if (@warn) {
+        $self->{warning_hash}{$idx} = \@warn ;
+        warn(map { "Warning in '".$self->location."': $_\n"} @warn) ;
+    }
+        
+    return scalar @error ? 0 : 1 ;
 }
 
+
+
 #internal
 sub check_follow_keys_from {
-    my ($self,$idx) = @_ ; 
+    my ($self,$idx,$error) = @_ ; 
 
     my $followed = $self->safe_typed_grab('follow_keys_from') ;
-    if ($followed->exists($idx)) {
-	return 1;
-    }
+    return if $followed->exists($idx) ;
 
-    $self->{error} = ["key '$idx' does not exists in '".$followed->name 
-		      . "'. Expected '"
-		      . join("', '", $followed->get_all_indexes)
-		      . "'"
-		     ] ;
-    return 0 ;
+    push @$error, "key '$idx' does not exists in '".$followed->name 
+              . "'. Expected '" . join("', '", $followed->get_all_indexes) . "'" ;
 }
 
 #internal
 sub check_allow_keys {
-    my ($self,$idx) = @_ ; 
+    my ($self,$idx,$error) = @_ ; 
 
     my $ok = grep { $_ eq $idx } @{$self->{allow_keys}} ;
 
-    return 1 if $ok ;
+    push @$error, "Unexpected key '$idx'. Expected '".
+                      join("', '",@{$self->{allow_keys}} ). "'" 
+        unless $ok ;
+}
 
-    $self->{error} = ["Unexpected key '$idx'. Expected '".
-		      join("', '",@{$self->{allow_keys}} ). "'"]   ;
-    return 0 ;
+#internal
+sub check_allow_keys_matching {
+    my ($self,$idx,$error) = @_ ; 
+    my $match = $self->{allow_keys_matching} ;
+
+    push @$error,"Unexpected key '$idx'. Key must match $match" 
+        unless $idx =~ /$match/;
 }
 
 #internal
 sub check_allow_keys_from {
-    my ($self,$idx) = @_ ; 
+    my ($self,$idx,$error) = @_ ; 
 
     my $from = $self->safe_typed_grab('allow_keys_from');
     my $ok = grep { $_ eq $idx } $from->get_all_indexes ;
 
-    return 1 if $ok ;
+    return if $ok ;
 
-    $self->{error} = ["key '$idx' does not exists in '"
-		      . $from->name 
-		      . "'. Expected '"
-		      . join( "', '", $from->get_all_indexes). "'" ] ;
+    push @$error, "key '$idx' does not exists in '"
+                      . $from->name 
+                      . "'. Expected '"
+                      . join( "', '", $from->get_all_indexes). "'"  ;
 
-    return 0 ;
 }
 
+sub check_warn_if_key_match {
+    my ($self,$idx,$error,$warn) = @_ ; 
+    my $re = $self->{warn_if_key_match} ;
+
+    push @$warn, "key '$idx' should not match $re\n" if $idx =~ /$re/ ;
+}
+
+sub check_warn_unless_key_match {
+    my ($self,$idx,$error,$warn) = @_ ; 
+    my $re = $self->{warn_unless_key_match} ;
+
+    push @$warn, "key '$idx' should match $re\n" unless $idx =~ /$re/;
+}
 
 =head1 Informations management
 
-=head2 fetch_with_id ( index )
+=head2 fetch_with_id ( index => $idx , [ check => 'no' ])
 
-Fetch the collected element held by the hash or list.
+Fetch the collected element held by the hash or list. Index check is 'yes' by default.
+Can be called with one parameter: idx.
 
 =cut
 
 sub fetch_with_id {
-    my ($self,$idx) = @_ ;
+    my $self = shift ;
+    my %args = @_ > 1 ? @_ : ( index => shift ) ;
+    my $check = $self->_check_check($args{check}) ;    
+    my $idx = $args{index} ;
 
     $self->warp 
       if ($self->{warp} and @{$self->{warp_info}{computed_master}});
 
     my $ok = $self->check($idx) ;
 
-    if ($ok) {
-	$self->auto_vivify($idx) unless $self->_defined($idx) ;
+    if ($ok or $check eq 'no') {
+        $self->auto_vivify($idx) unless $self->_defined($idx) ;
         return $self->_fetch_with_id($idx) ;
       }
-    elsif ($self->instance->get_value_check('fetch')) {
+    else {
         Config::Model::Exception::WrongValue 
-	    -> throw (
-		      error => join("\n\t",@{$self->{error}}),
-		      object => $self
-		     ) ;
+            -> throw (
+                      error => join("\n\t",@{$self->{error}}),
+                      object => $self
+                     ) ;
     }
 
     return ;
 }
 
-=head2 get( path,  [ custom | preset | standard | default ])
+=head2 get( path => ..., mode => ... ,  check => ... , get_obj => 1|0, autoadd => 1|0)
 
 Get a value from a directory like path.
 
@@ -791,12 +820,21 @@ Get a value from a directory like path.
 
 sub get {
     my $self = shift ;
-    my $path = shift ;
+    my %args = @_ > 1 ? @_ : ( path => $_[0] ) ;
+    my $path = delete $args{path} ;
+    my $autoadd = 1 ;
+    $autoadd = $args{autoadd} if defined $args{autoadd};
+    my $get_obj = delete $args{get_obj} || 0 ;
     $path =~ s!^/!! ;
     my ($item,$new_path) = split m!/!,$path,2 ;
-    my $obj = $self->fetch_with_id($item) ;
-    return $obj if ($obj->get_type ne 'leaf' and not defined $new_path) ;
-    return $obj->get($new_path,@_) ;
+
+    return unless ($self->exists($item) or $autoadd) ;
+
+    $logger->debug("get: path $path, item $item");
+    
+    my $obj = $self->fetch_with_id(index => $item, %args) ;
+    return $obj if (($get_obj or $obj->get_type ne 'leaf') and not defined $new_path) ;
+    return $obj->get(path => $new_path,get_obj => $get_obj, %args) ;
 }
 
 =head2 set( path, value )
@@ -813,35 +851,6 @@ sub set {
     return $self->fetch_with_id($item)->set($new_path,@_) ;
 }
 
-=head2 move ( from_index, to_index )
-
-Move an element within the hash or list.
-
-=cut
-
-sub move {
-    my ($self,$from, $to) = @_ ;
-
-    my $moved = $self->fetch_with_id($from) ;
-    $self->_delete($from);
-
-    my $ok = $self->check($to) ;
-    if ($ok) {
-	$self->_store($to, $moved) ;
-	$moved->index_value($to) ;
-    }
-    else {
-	# restore moved item where it came from
-	$self->_store($from, $moved) ;
-	if ($self->instance->get_value_check('fetch')) {
-	    Config::Model::Exception::WrongValue 
-		-> throw (
-			  error => join("\n\t",@{$self->{error}}),
-			  object => $self
-			 ) ;
-	}
-    }
-}
 
 =head2 copy ( from_index, to_index )
 
@@ -858,18 +867,22 @@ sub copy {
     my $ok = $self->check($to) ;
 
     if ($ok && $self->{cargo}{type} eq 'leaf') {
-	$self->fetch_with_id($to)->store($from_obj->fetch()) ;
+        $logger->trace("AnyId: copy leaf value from ".$self->name." $from to $to") ;
+        $self->fetch_with_id($to)->store($from_obj->fetch()) ;
     }
     elsif ( $ok ) {
-	# node object 
-	$self->fetch_with_id($to)->copy_from($from_obj) ;
+        # node object 
+        $logger->trace("AnyId: deep copy node from ".$self->name) ;
+        my $target = $self->fetch_with_id($to);
+        $logger->trace("AnyId: deep copy node to ".$target->name) ;
+        $target->copy_from($from_obj) ;
     }
-    elsif ($self->instance->get_value_check('fetch')) {
-	Config::Model::Exception::WrongValue 
-	    -> throw (
-		      error => join("\n\t",@{$self->{error}}),
-		      object => $self
-		     ) ;
+    else {
+        Config::Model::Exception::WrongValue 
+            -> throw (
+                      error => join("\n\t",@{$self->{error}}),
+                      object => $self
+                     ) ;
     }
 }
 
@@ -885,12 +898,13 @@ sub fetch_all {
     return map { $self->fetch_with_id($_) ;} @keys ;
 }
 
-=head2 fetch_all_values( [ custom | preset | standard | default ] )
+=head2 fetch_all_values( mode => ..., check => ...)
 
 Returns an array containing all defined values held by the hash or
-list. (undefined values are simply discarded)
+list. (undefined values are simply discarded). This method is only 
+valid for hash or list containing leaves.
 
-With a parameter, this method will return either:
+With C<mode> parameter, this method will return either:
 
 =over
 
@@ -916,28 +930,30 @@ The default value (defined by the configuration model)
 
 sub fetch_all_values {
     my $self = shift ;
-    my $mode = shift ;
-
+    my %args = @_ > 1 ? @_ : ( mode => shift ) ;
+    my $mode = $args{mode};
+    my $check = $self->_check_check($args{check}) ;
+    
     my @keys  = $self->get_all_indexes ;
 
     if ($self->{cargo}{type} eq 'leaf') {
-	return grep {defined $_} 
-	  map { $self->fetch_with_id($_)->fetch($mode) ;} @keys ;
+        return grep {defined $_} 
+          map { $self->fetch_with_id($_)->fetch(check => $check, mode => $mode) ;} @keys ;
     }
     else {
-	my $info = "current keys are '".join("', '",@keys)."'." ;
-	if ($self->{cargo}{type} eq 'node') {
-	    $info .= "config class is ".
-	      $self->fetch_with_id($keys[0])->config_class_name ;
-	}
-	Config::Model::Exception::WrongType
-	    ->throw(
-		    object => $self,
-		    function => 'fetch_all_values',
-		    got_type => $self->{cargo}{type},
-		    expected_type => 'leaf',
-		    info => $info,
-		   )
+        my $info = "current keys are '".join("', '",@keys)."'." ;
+        if ($self->{cargo}{type} eq 'node') {
+            $info .= "config class is ".
+              $self->fetch_with_id($keys[0])->config_class_name ;
+        }
+        Config::Model::Exception::WrongType
+            ->throw(
+                    object => $self,
+                    function => 'fetch_all_values',
+                    got_type => $self->{cargo}{type},
+                    expected_type => 'leaf',
+                    info => $info,
+                   )
     }
 }
 
@@ -951,11 +967,23 @@ are sorted alphabetically, except for ordered hashed.
 sub get_all_indexes {
     my $self = shift;
     $self->create_default if (   defined $self->{default_keys}
-			      or defined $self->{default_with_init}
-			      or defined $self->{follow_keys_from});
+                              or defined $self->{default_with_init}
+                              or defined $self->{follow_keys_from});
     return $self->_get_all_indexes ;
 }
 
+=head2 children 
+
+Like get_all_indexes. This method is
+polymorphic for all non-leaf objects of the configuration tree.
+
+=cut
+
+sub children {
+    my $self = shift ;
+    return $self->get_all_indexes ;
+}
+
 
 # auto vivify must create according to cargo}{type
 # node -> Node or user class
@@ -986,63 +1014,63 @@ sub auto_vivify {
     my $cargo_type = delete $cargo_args{type} ;
 
     Config::Model::Exception::Model 
-	-> throw (
-		  object => $self,
-		  message => "unknown '$cargo_type' cargo type:  "
-		  ."in cargo_args. Expected "
-		  .join (' or ',keys %element_default_class)
-		 ) 
-	      unless defined $element_default_class{$cargo_type} ;
+        -> throw (
+                  object => $self,
+                  message => "unknown '$cargo_type' cargo type:  "
+                  ."in cargo_args. Expected "
+                  .join (' or ',keys %element_default_class)
+                 ) 
+              unless defined $element_default_class{$cargo_type} ;
 
     my $el_class = 'Config::Model::'
       . $element_default_class{$cargo_type} ;
 
     if (defined $class) {
-	Config::Model::Exception::Model 
-	    -> throw (
-		      object => $self,
-		      message => "$cargo_type class "
-		      ."cannot be overidden by '$class'"
-		     ) 
-	      unless $can_override_class{$cargo_type} ;
-	$el_class = $class;
+        Config::Model::Exception::Model 
+            -> throw (
+                      object => $self,
+                      message => "$cargo_type class "
+                      ."cannot be overidden by '$class'"
+                     ) 
+              unless $can_override_class{$cargo_type} ;
+        $el_class = $class;
     }
 
     if (not defined *{$el_class.'::new'}) {
-	my $file = $el_class.'.pm';
-	$file =~ s!::!/!g;
-	require $file ;
+        my $file = $el_class.'.pm';
+        $file =~ s!::!/!g;
+        require $file ;
     }
 
     my %common_args = (
-		       element_name => $self->{element_name},
-		       index_value  => $idx,
-		       instance     => $self->{instance} ,
-		      );
+                       element_name => $self->{element_name},
+                       index_value  => $idx,
+                       instance     => $self->{instance} ,
+                      );
 
     my $item ;
 
     # check parameters passed by the user
     if ($cargo_type eq 'node') {
-	Config::Model::Exception::Model 
-	    -> throw (
-		      object => $self,
-		      message => "missing 'cargo->config_class_name' "
-		      ."parameter",
-		     ) 
-	      unless defined $self->{config_class_name} ;
+        Config::Model::Exception::Model 
+            -> throw (
+                      object => $self,
+                      message => "missing 'cargo->config_class_name' "
+                      ."parameter",
+                     ) 
+              unless defined $self->{config_class_name} ;
 
-	$item = $self->{parent} 
-	  -> new( %common_args ,
-		  config_class_name => $self->{config_class_name},
-		  %cargo_args) ;
+        $item = $self->{parent} 
+          -> new( %common_args ,
+                  config_class_name => $self->{config_class_name},
+                  %cargo_args) ;
     }
     else {
-	weaken($common_args{id_owner} =  $self) ;
-	$item = $el_class->new( %common_args,
-				parent => $self->{parent} ,
-				instance => $self->{instance} ,
-				%cargo_args) ;
+        weaken($common_args{id_owner} =  $self) ;
+        $item = $el_class->new( %common_args,
+                                parent => $self->{parent} ,
+                                instance => $self->{instance} ,
+                                %cargo_args) ;
     }
 
     $self->_store($idx,$item) ;
@@ -1092,6 +1120,7 @@ sub delete {
     $self->warp 
       if ($self->{warp} and @{$self->{warp_info}{computed_master}});
 
+    delete $self->{warning_hash}{$idx}  ;
     return $self->_delete($idx);
   }
 
@@ -1106,7 +1135,7 @@ sub clear {
 
     $self->warp 
       if ($self->{warp} and @{$self->{warp_info}{computed_master}});
-
+    $self->{warning_hash} = {} ;
     $self->_clear;
   }
 
@@ -1121,11 +1150,11 @@ sub clear_values {
 
     my $ct = $self->get_cargo_type ;
     Config::Model::Exception::User
-	-> throw (
-		  object => $self,
-		  message => "clear_values() called on non leaf cargo type: '$ct'"
-		 ) 
-	  if $ct ne 'leaf';
+        -> throw (
+                  object => $self,
+                  message => "clear_values() called on non leaf cargo type: '$ct'"
+                 ) 
+          if $ct ne 'leaf';
 
     $self->warp 
       if ($self->{warp} and @{$self->{warp_info}{computed_master}});
@@ -1133,6 +1162,20 @@ sub clear_values {
     map {$self->fetch_with_id($_)->store(undef)} $self->get_all_indexes ;
   }
 
+=head2 warning_msg ( [index] )
+
+Returns warnings concerning indexes of this hash. 
+Without parameter, returns a hash ref or undef. With an index, return the warnings
+concerning this index or undef.
+
+=cut
+
+sub warning_msg {
+    my ($self,$idx) = @_ ;
+    return unless scalar %{$self->{warning_hash}};
+    return defined $idx ? $self->{warning_hash}{$idx} : $self->{warning_hash} ;
+}
+
 1;
 
 __END__
