@@ -1,12 +1,12 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 #    Copyright (c) 2005-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
@@ -26,12 +26,12 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::AnyThing;
+BEGIN {
+  $Config::Model::AnyThing::VERSION = '1.226';
+}
 use Scalar::Util qw(weaken);
 use Carp;
 use strict;
-our $VERSION="1.202";
-
-# use vars qw($VERSION);
 
 =head1 NAME
 
@@ -39,7 +39,7 @@ Config::Model::AnyThing - Base class for configuration tree item
 
 =head1 VERSION
 
-version 1.205
+version 1.226
 
 =head1 SYNOPSIS
 
@@ -87,14 +87,14 @@ sub index_value {
     return $self->{index_value} ;
 }
 
-=head2 get_type()
+=head2 get_container_type()
 
 Returns the type (e.g. C<list> or C<hash> or C<leaf> or C<node> or
 C<warped_node>) of the element containing this object. 
 
 =cut 
 
-sub get_type {
+sub get_container_type {
     my $self = shift;
     my $p = $self->parent ;
     return defined $p ? $p->element_type($self->element_name)
@@ -195,11 +195,18 @@ Annotation is a way to store miscellaneous information associated to
 each node. (Yeah... comments) These comments will be saved outside of
 the configuration file and restored the next time the command is run.
 
+=head2 annotation( [ note1, [ note2 , ... ] ] )
+
+Without argument, return a string containing the object's annotation.
+
+With several arguments, join the arguments with "\n", store the annotations 
+and return the resulting string.
+
 =cut
 
 sub annotation {
     my $self = shift ;
-    $self->{annotation} = join("\n",@_) if @_;
+    $self->{annotation} = join("\n", grep (defined $_,@_)) if @_;
     return $self->{annotation} ;
 }
 
@@ -290,8 +297,8 @@ considered when going up the tree.
 
 sub grab {
     my $self = shift ;
-    my ($step,$strict,$autoadd, $type, $grab_non_available)
-      = (undef, 1, 1, undef, 0 ) ;
+    my ($step,$strict,$autoadd, $type, $grab_non_available,$check)
+      = (undef, 1, 1, undef, 0, 'yes' ) ;
     if ( @_ > 1 ) {
 	my %args = @_;
 	$step    = $args{step};
@@ -300,6 +307,7 @@ sub grab {
 	$grab_non_available = $args{grab_non_available} 
 	  if defined $args{grab_non_available};
 	$type    = $args{type} ; # node, leaf or undef
+	$check = $self->_check_check($args{check}) ;
     }
     elsif (@_ == 1) {
 	$step = shift ;
@@ -388,7 +396,7 @@ sub grab {
 	}
 
         my ($name, $action, $arg) 
-	  = ($cmd =~ /(\w+)(?:(:)((?:"[^\"]*")|(?:[\w:\/\.\-]+)))?/);
+	  = ($cmd =~ /([\-\w]+)(?:(:)((?:"[^\"]*")|(?:[\w:\/\.\-\+]+)))?/);
 
 	if (defined $arg and $arg =~ /^"/ and $arg =~ /"$/) {
 	    $arg =~ s/^"// ; # remove leading quote
@@ -427,7 +435,8 @@ sub grab {
 	   last ;
 	}
 
-	my $next_obj = $obj->fetch_element($name,'master',$grab_non_available) ;
+	my $next_obj = $obj->fetch_element( name => $name,
+	    experience => 'master', check => $check, accept_hidden => $grab_non_available) ;
 
 	# create list or hash element only if autoadd is true
         if (defined $action and $autoadd == 0
@@ -594,6 +603,18 @@ sub dump_as_data {
     $dumper->dump_as_data(node => $self, @_) ;
 }
 
+# hum, check if the check information is valid
+sub _check_check {
+    my $self = shift ;
+    my $p = shift ;
+
+    return 'yes' if not defined $p or $p eq '1' or $p eq 'yes';
+    return 'no'  if $p eq '0' or $p eq 'no' ;
+    return $p    if $p eq 'skip' ;
+
+    croak "Internal error: Unvalid check value: $p" ;
+}
+
 
 1;
 
