@@ -1,14 +1,14 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 
-#    Copyright (c) 2006-2007 Dominique Dumont.
+#    Copyright (c) 2006-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -27,15 +27,16 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Describe;
+BEGIN {
+  $Config::Model::Describe::VERSION = '1.226';
+}
 use Carp;
 use strict;
-our $VERSION="1.201";
 use warnings ;
 
 use Config::Model::Exception ;
 use Config::Model::ObjTreeScanner ;
 
-# use vars qw($VERSION);
 
 =head1 NAME
 
@@ -43,7 +44,7 @@ Config::Model::Describe - Provide a description of a node element
 
 =head1 VERSION
 
-version 1.205
+version 1.226
 
 =head1 SYNOPSIS
 
@@ -126,11 +127,12 @@ sub describe {
     my $desc_node = delete $args{node} 
       || croak "describe: missing 'node' parameter";
     my $element = delete $args{element} ; # optional
+    my $check = delete $args{check} || 'yes' ;
 
     my $std_cb = sub {
         my ( $scanner, $data_r, $obj, $element, $index, $value_obj ) = @_;
 
-	my $value = $value_obj->fetch ;
+	my $value = $value_obj->fetch (check => $check) ;
         $value = '"' . $value . '"' if defined $value and $value =~ /\s/;
 
 	#print "DEBUG: std_cb on $element, idx $index, value $value\n";
@@ -162,7 +164,7 @@ sub describe {
         }
         else {
             push @$data_r , [ $element,
-			    join( ',', $list_obj->fetch_all_values ),
+			    join( ',', $list_obj->fetch_all_values(check => 'no' )),
 			    'list','' ];
         }
     };
