@@ -1,12 +1,12 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 
 #    Copyright (c) 2008 Dominique Dumont.
 #
@@ -28,13 +28,14 @@
 #    02110-1301 USA
 
 package Config::Model::SimpleUI ;
+BEGIN {
+  $Config::Model::SimpleUI::VERSION = '1.226';
+}
 
 use Carp;
 use strict ;
-our $VERSION="1.201";
 use warnings ;
 
-# use vars qw($VERSION);
 
 =head1 NAME
 
@@ -42,7 +43,7 @@ Config::Model::SimpleUI - Simple interface for Config::Model
 
 =head1 VERSION
 
-version 1.205
+version 1.226
 
 =head1 SYNOPSIS
 
@@ -162,9 +163,7 @@ my $ll_sub = sub {
     my $obj = $self->{current_node} ;
 
     my $i = $self->{current_node}->instance;
-    $i->push_no_value_check('fetch') ;
-    my $res = $obj->describe(element => $elt) ;
-    $i->pop_no_value_check;
+    my $res = $obj->describe(element => $elt, check =>'no') ;
     return $res ; 
 } ;
 
@@ -216,17 +215,13 @@ my %run_dispatch =
    ls => sub { 
        my $self = shift ;
        my $i = $self->{current_node}->instance;
-       $i->push_no_value_check('fetch') ;
        my @res = $self->{current_node}->get_element_name ;
-       $i->pop_no_value_check;
        return join('  ',@res) ;
    },
    dump => sub { 
        my $self = shift ;
        my $i = $self->{current_node}->instance;
-       $i->push_no_value_check('fetch') ;
        my @res = $self->{current_node}-> dump_tree(full_dump => 1);
-       $i->pop_no_value_check;
        return join('  ',@res) ;
    },
    delete => sub {
