@@ -1,6 +1,12 @@
-# $Author: ddumont $
-# $Date: 2008-05-17 17:58:38 +0200 (Sat, 17 May 2008) $
-# $Revision: 669 $
+#
+# This file is part of Config-Model
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
+# This is free software, licensed under:
+#
+#   The GNU Lesser General Public License, Version 2.1, February 1999
+#
 
 #    Copyright (c) 2006-2008 Dominique Dumont.
 #
@@ -22,6 +28,9 @@
 #    02110-1301 USA
 
 package Config::Model::TermUI ;
+BEGIN {
+  $Config::Model::TermUI::VERSION = '1.226';
+}
 
 use Carp;
 use strict ;
@@ -29,15 +38,17 @@ use warnings ;
 
 use Term::ReadLine;
 
-use vars qw($VERSION);
 use base qw/Config::Model::SimpleUI/ ;
 
-$VERSION = sprintf "1.%04d", q$Revision: 669 $ =~ /(\d+)/;
 
 =head1 NAME
 
 Config::Model::TermUI - Provides Config::Model UI  la Term::ReadLine
 
+=head1 VERSION
+
+version 1.226
+
 =head1 SYNOPSIS
 
  my $model = Config::Model -> new ;
@@ -178,9 +189,7 @@ sub completion {
 
     if ( $space_idx > 0 and defined $completion_dispatch{$main}) {
 	my $i = $self->{current_node}->instance;
-	$i->push_no_value_check('fetch') ;
 	return $completion_dispatch{$main}->($self,$text,$line,$start) ;
-	$i->pop_no_value_check;
     }
     elsif (not $cmd) {
 	return $self->simple_ui_commands() ;
@@ -312,4 +321,4 @@ L<Config::Model>,
 L<Config::Model::Instance>,
 L<Config::Model::Node>,
 
-=cut
+=cut
\ No newline at end of file
