@@ -1,14 +1,14 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 
-#    Copyright (c) 2005-2009 Dominique Dumont.
+#    Copyright (c) 2005-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -27,8 +27,10 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::WarpedThing ;
+BEGIN {
+  $Config::Model::WarpedThing::VERSION = '1.226';
+}
 use strict;
-our $VERSION="1.201";
 use Scalar::Util qw(weaken) ;
 use Data::Dumper ;
 use Config::Model::ValueComputer ;
@@ -38,7 +40,6 @@ use Carp;
 
 use warnings FATAL => qw(all);
 
-# use vars qw($VERSION) ;
 
 use base qw/Config::Model::AnyThing/ ;
 
@@ -50,7 +51,7 @@ Config::Model::WarpedThing - Base class for warped classes
 
 =head1 VERSION
 
-version 1.205
+version 1.226
 
 =head1 SYNOPSIS
 
@@ -356,7 +357,7 @@ sub warp {
         get_logger("Tree::Element::Warped")
 	  ->debug( "Warp called with value '", 
 		   defined $value ? $value : '<undef>',
-		   "'name $warp_name");
+		   "' name $warp_name");
         $warp_value_set->{$warp_name} = $value ;
     }
 
@@ -439,7 +440,7 @@ sub _do_warp {
     # try all boolean expression with warp_value_set to get the
     # correct rule
 
-    my $found_rule ;
+    my $found_rule = {};
     my $found_bool ='' ; # this variable may be used later in error message
 
     foreach my $bool_expr (@$rules) {
@@ -447,7 +448,7 @@ sub _do_warp {
 	my $res = $self -> compute_bool( $bool_expr );
 	next unless $res ;
 	$found_bool = $bool_expr ;
-	$found_rule = $self->{warp_info}{rule_hash}{$bool_expr} ;
+	$found_rule = $self->{warp_info}{rule_hash}{$bool_expr} || {};
 	$logger->debug("_do_warp found rule for '$bool_expr':\n", 
 		       Data::Dumper->Dump ([$found_rule],['found_rule']));
 	last;
@@ -456,20 +457,21 @@ sub _do_warp {
     if ($logger->is_info) {
         my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set ;
 
-        $logger->info("warp called from '$found_bool' on '",$self->name,
+        $logger->info("_do_warp: warp called from '$found_bool' on '",$self->name,
 		      "' with elements '",join("','",@warp_str),
-		      "', warp rule is ", (defined $found_rule ? "" : 'not ') , 
+		      "', warp rule is ", (scalar %$found_rule ? "" : 'not ') , 
 		      "found");
     }
 
-    $logger->debug("warp_them: call set_properties on '",$self->name,"'");
+    $logger->debug("warp_them: call set_properties on '",$self->name,"' with ",
+	Data::Dumper->Dump ([$found_rule],['found_rule']));
 
     eval { $self->set_properties(%$found_rule) ; };
 
     if ($@) {
         my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set ;
 	my $e = $@ ;
-	my $msg = $e ? $e->error : '' ;
+	my $msg = ref $e ? $e->as_string : $e ;
 	Config::Model::Exception::Model
 	    -> throw (
 		      object => $self,
